"""Mapping data types

A mapping is a set of *(key, value)* pairs.  A mapping data type is a
container that associates zero or more objects with each item of some
collection.

Despite the fact that a mapping is fundamentally a set,
:class:`collections.Mapping` does not support the API of
:class:`collections.Set`.

"""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from ._misc import *
