"""Set data types

A set is a category of which every object is either a member or not.  A
set data type is a container whose items are distinct and unordered.

"""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from ._misc import *
from ._universalizable import *
